module TaggableController
  extend ActiveSupport::Concern

  included do
    before_action :authenticate_user!, only: [:tagging, :untagging]
  end

  # POST /v1/learning_objects/1/tagging
  # POST /v1/learning_objects/1/tagging.json
  def tagging
    if current_user.tag(taggable, with: taggable.owner_tags_on(current_user, :tags) << tag_params[:name], on: :tags)
      render json: @taggable, status: :created
    else
      render json: @taggable.errors, status: :unprocessable_entity
    end
  end

  # DELETE /v1/learning_objects/1/untagging
  # DELETE /v1/learning_objects/1/untagging.json
  def untagging
    if current_user.tag(taggable, with: taggable.owner_tags_on(current_user, :tags).delete(tag_params[:name]), on: :tags)
      render json: @taggable, status: :ok
    else
      render json: @taggable.errors, status: :unprocessable_entity
    end
  end

  protected

  def taggable
    raise NotImplementedError
  end

  def tag_params
    params.require(:tag).permit(:name)
  end
end
